<?php
namespace Teknisa\PAR\Service;

use Teknisa\Model\Academico\Entities\Endeprospect;
use Teknisa\Model\Crm\Entities\Prospect;
use Teknisa\Model\Geral\Entities\Cliente;
use Teknisa\Model\Geral\Entities\Endeclie;
use Teknisa\Model\Geral\Entities\Endeforn;
use Teknisa\Model\Geral\Entities\Enderecoparc;
use Teknisa\Model\Geral\Entities\Fornecedor;
use Teknisa\Model\Geral\Entities\Vendedor;
use Teknisa\Model\Geral\Entities\Endevende;

class EnderecoParceiroService {

    protected $entityManager;
    protected $environment;
    protected $newCode;

    public function __construct($entityManager, $environment, $newCode) {
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->newCode = $newCode;
    }

    /*
     *
     *  Funções de manipulação de dados
     *
     */
    public function saveEnderecoParceiro($row) {
        $this->validateEnderecoParceiro($row);

        $nrorg = $this->environment->getNrorgTrab();
        $operador = $this->environment->getCdOperador();
        $currentTime = $this->environment->getCurrentTime();

        $nrparcnegocio  = $row["NRPARCNEGOCIO"];
        $cdtipoendereco = $row["CDTIPOENDERECO"];

        $enderecoparcRepository = $this->entityManager->getRepository(Enderecoparc::class);

        // Utilizado caso o tipo de endereço seja alterado, excluindo o antigo e inserindo um novo
        if (!$row["__is_new"] && $row["tipoEnderecoWasEdited"]) {
            $rowToDelete = clone $row;
            $rowToDelete['CDTIPOENDERECO'] = $rowToDelete['CDTIPOENDERECOANTIGO'];
            $this->removeEnderecoParceiro($rowToDelete);
        }

        $this->entityManager->beginTransaction();
        try {
            $enderecoparc = $enderecoparcRepository->findOneBy(array(
                "nrparcnegocio" => $nrparcnegocio,
                "cdtipoendereco" => $cdtipoendereco,
                "nrorg" => $nrorg
            ));
            if ($enderecoparc === null) {
                $enderecoparc = new Enderecoparc();
                $enderecoparc->setNrorg($nrorg);
                $enderecoparc->setNrenderecoparc($this->newCode->getCodeByTableName('ENDERECOPARC', $nrorg, 1, 10, true));
                $enderecoparc->setNrparcnegocio($nrparcnegocio);
                $enderecoparc->setDtinclusao($currentTime);
                $enderecoparc->setCdoperinclusao($operador);
                $enderecoparc->setNrorginclusao($nrorg);
                $enderecoparc->setIdativo("S");
                $row["NRENDERECOPARC"] = $enderecoparc->getNrenderecoparc();
            }
            else {
                $enderecoparc->setDtultatu($currentTime);
                $enderecoparc->setCdoperultatu($operador);
                $enderecoparc->setNrorgultatu($nrorg);
            }
            $enderecoparc->setCdtipoendereco($cdtipoendereco);
            $enderecoparc->setCdpais($row["CDPAIS"]);
            $enderecoparc->setSgestado($row["SGESTADO"]);
            $enderecoparc->setCdmunicipio($row["CDMUNICIPIO"]);
            $enderecoparc->setDsendereco($row["DSENDERECO"]);
            $enderecoparc->setCdbairro(!empty($row["CDBAIRRO"]) ? $row["CDBAIRRO"] : null);
            $enderecoparc->setCdlogradouro(!empty($row["CDLOGRADOURO"]) ? $row["CDLOGRADOURO"] : null);
            $enderecoparc->setNrimovelendereco(!empty($row["NRIMOVELENDERECO"]) ? $row["NRIMOVELENDERECO"] : null);
            $enderecoparc->setNmbairroendereco(!empty($row["NMBAIRROENDERECO"]) ? $row["NMBAIRROENDERECO"] : null);
            $enderecoparc->setDscompleendereco(!empty($row["DSCOMPLEENDERECO"]) ? $row["DSCOMPLEENDERECO"] : null);
            $enderecoparc->setNrcependereco(!empty($row["NRCEPENDERECO"]) ? $row["NRCEPENDERECO"] : null);
            $enderecoparc->setDsreferenciaende(!empty($row["DSREFERENCIAENDE"]) ? $row["DSREFERENCIAENDE"] : null);

            
            $this->entityManager->persist($enderecoparc);
            $this->entityManager->flush();

            $this->replicaEnderecoParceiro($enderecoparc, $row);

            $this->entityManager->commit();
        } catch(\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }

        return $enderecoparc->getNrenderecoparc();
    }

    public function replicaEnderecoParceiro(Enderecoparc $enderecoparc, $row) {
        $nrorg = $this->environment->getNrorgTrab();
        $operador = $this->environment->getCdOperador();
        $currentTime = $this->environment->getCurrentTime();

        $tipoEndereco = array('PRINCIPAL' => 'P',
                               'COBRANCA' => 'C',
                               'COMERCIAL' => 'M',
                               'ENTREGA' => 'T',
                               'EXTERIOR' => 'E',
                               'EXPORTACAO' => 'X',
                               'OUTROS' => 'O');
        $tiposParceiro = $row["TIPODEPARCEIRO"];
        
        if (is_array($tiposParceiro)){
             $tiposParceiroAux = $tiposParceiro;
        }
        else {
            $tiposParceiroAux = array();
            $tiposParceiroAux[] = $tiposParceiro;
        }
        
        $nrenderecoparc = $enderecoparc->getNrenderecoparc();
        $cdtipoendereco = $enderecoparc->getCdtipoendereco();
        $nrparcnegocio = $enderecoparc->getNrparcnegocio();
        $dsendereco = substr($enderecoparc->getDsendereco(), 0, 50);
        $idtpendereco = (isset($tipoEndereco[$cdtipoendereco])) ? $tipoEndereco[$cdtipoendereco] : false;
        if ($enderecoparc->getNrimovelendereco() !== null) {
            $dsendereco .= ", " . $enderecoparc->getNrimovelendereco();
        }

        $endeclieRepository = $this->entityManager->getRepository(Endeclie::class);
        $clienteRepository = $this->entityManager->getRepository(Cliente::class);
        $endefornRepository = $this->entityManager->getRepository(Endeforn::class);
        $fornecedorRepository = $this->entityManager->getRepository(Fornecedor::class);
        $endevendeRepository = $this->entityManager->getRepository(Endevende::class);
        $vendedorRepository = $this->entityManager->getRepository(Vendedor::class);
        $endeprospectRepository = $this->entityManager->getRepository(Endeprospect::class);
        $prospectRepository = $this->entityManager->getRepository(Prospect::class);


        if ($idtpendereco) {
            if (in_array('CLIENTE', $tiposParceiroAux)) {
                $endeclie = $endeclieRepository->findOneBy(array(
                    "nrenderecoparc" => $nrenderecoparc,
                    "nrorg" => $nrorg
                ));
                if ($endeclie === null) {
                    $cliente = $clienteRepository->findOneBy(array(
                        "nrparcnegocio" => $nrparcnegocio,
                        "nrorg" => $nrorg
                    ));
                    if ($cliente !== null) {
                        $endeclie = new Endeclie();
                        $endeclie->setNrorg($nrorg);
                        $endeclie->setNrenderecoparc($nrenderecoparc);
                        $endeclie->setCdcliente($cliente->getCdcliente());
                        $endeclie->setidtpendeclie($idtpendereco);
                        $endeclie->setCdpais($enderecoparc->getCdpais());
                        $endeclie->setSgestado($enderecoparc->getSgestado());
                        $endeclie->setCdmunicipio($enderecoparc->getCdmunicipio());
                        $endeclie->setDsendeclie(substr($dsendereco, 0, 50));
                        $endeclie->setCdbairro($enderecoparc->getCdbairro());
                        $endeclie->setDscompendcli(substr($enderecoparc->getDscompleendereco(), 0, 200));
                        $endeclie->setNmbairclie(substr($enderecoparc->getNmbairroendereco(), 0, 50));
                        $endeclie->setNrcepclie($enderecoparc->getNrcependereco());
                        $endeclie->setIdativo('S');
                        $endeclie->setDtinclusao($currentTime);
                        $endeclie->setCdoperinclusao($operador);
                        $endeclie->setNrorginclusao($nrorg);
                        $this->entityManager->persist($endeclie);
                        $this->entityManager->flush();
                    }
                }
                else {
                    $endeclie->setidtpendeclie($idtpendereco);
                    $endeclie->setCdpais($enderecoparc->getCdpais());
                    $endeclie->setSgestado($enderecoparc->getSgestado());
                    $endeclie->setCdmunicipio($enderecoparc->getCdmunicipio());
                    $endeclie->setDsendeclie(substr($dsendereco, 0, 50));
                    $endeclie->setCdbairro($enderecoparc->getCdbairro());
                    $endeclie->setDscompendcli(substr($enderecoparc->getDscompleendereco(), 0, 200));
                    $endeclie->setNmbairclie(substr($enderecoparc->getNmbairroendereco(), 0, 50));
                    $endeclie->setNrcepclie($enderecoparc->getNrcependereco());
                    $endeclie->setDtultatu($currentTime);
                    $endeclie->setCdoperultatu($operador);
                    $endeclie->setNrorgultatu($nrorg);
                    $this->entityManager->persist($endeclie);
                    $this->entityManager->flush();
                }
            }else{
                $endeclie = $endeclieRepository->findOneBy(array(
                    "nrenderecoparc" => $nrenderecoparc,
                    "nrorg" => $nrorg
                ));
                if ($endeclie !== null){
                    $endeclie->setIdativo('N');
                     $this->entityManager->persist($endeclie);
                     $this->entityManager->flush();
                }
            
            }

            if ($idtpendereco != "M") {
                if (in_array('PROSPECT', $tiposParceiroAux)) {
                    $endeprospect = $endeprospectRepository->findOneBy(array(
                        "nrenderecoparc" => $nrenderecoparc,
                        "nrorg" => $nrorg
                    ));
                    if ($endeprospect === null) {
                        $prospect = $prospectRepository->findOneBy(array(
                            "nrparcnegocio" => $nrparcnegocio,
                            "nrorg" => $nrorg
                        ));
                        if ($prospect !== null) {
                            $endeprospect = new Endeprospect();
                            $endeprospect->setNrorg($nrorg);
                            $endeprospect->setNrenderecoparc($nrenderecoparc);
                            $endeprospect->setCdprospect($prospect->getCdprospect());
                            $endeprospect->setIdtpendepros($idtpendereco);
                            $endeprospect->setCdpais($enderecoparc->getCdpais());
                            $endeprospect->setSgestado($enderecoparc->getSgestado());
                            $endeprospect->setCdmunicipio($enderecoparc->getCdmunicipio());
                            $endeprospect->setCdbairro($enderecoparc->getCdbairro());
                            $endeprospect->setDsendepros($enderecoparc->getDsendereco());
                            $endeprospect->setNrimovelendereco($enderecoparc->getNrimovelendereco());
                            $endeprospect->setNmbairpros($enderecoparc->getNmbairroendereco());
                            $endeprospect->setNrceppros($enderecoparc->getNrcependereco());
                            $endeprospect->setNrenderecoparc($enderecoparc->getNrenderecoparc());
                            $endeprospect->setIdativo('S');
                            $endeprospect->setDtinclusao($currentTime);
                            $endeprospect->setCdoperinclusao($operador);
                            $endeprospect->setNrorginclusao($nrorg);
                            $this->entityManager->persist($endeprospect);
                            $this->entityManager->flush();
                        }
                    }
                    else {
                        $endeprospect->setIdtpendepros($idtpendereco);
                        $endeprospect->setCdpais($enderecoparc->getCdpais());
                        $endeprospect->setSgestado($enderecoparc->getSgestado());
                        $endeprospect->setCdmunicipio($enderecoparc->getCdmunicipio());
                        $endeprospect->setCdbairro($enderecoparc->getCdbairro());
                        $endeprospect->setDsendepros($enderecoparc->getDsendereco());
                        $endeprospect->setNrimovelendereco($enderecoparc->getNrimovelendereco());
                        $endeprospect->setNmbairpros($enderecoparc->getNmbairroendereco());
                        $endeprospect->setNrceppros($enderecoparc->getNrcependereco());
                        $endeprospect->setDtultatu($currentTime);
                        $endeprospect->setCdoperultatu($operador);
                        $endeprospect->setNrorgultatu($nrorg);
                        $this->entityManager->persist($endeprospect);
                        $this->entityManager->flush();
                    }
                }
                else{
                    $endeprospect = $endeprospectRepository->findOneBy(array(
                        "nrenderecoparc" => $nrenderecoparc,
                        "nrorg" => $nrorg
                    ));
                    if ($endeprospect !== null){
                        $endeprospect->setIdativo('N');
                         $this->entityManager->persist($endeprospect);
                         $this->entityManager->flush();
                    }
                }
            }
        }
    }

    public function removeEnderecoParceiro($row) {
        $nrorg = $this->environment->getNrorgTrab();
        $nrenderecoparc = $row["NRENDERECOPARC"];
        $params = array(
            "nrenderecoparc" => $nrenderecoparc,
            "nrorg" => $nrorg
        );

        $this->entityManager->beginTransaction();
        try {
            
            $endeclie = $this->entityManager->getRepository(Endeclie::class)->findOneBy($params);
            if ($endeclie !== null) {
                $this->entityManager->remove($endeclie);
            }

            $endeforn = $this->entityManager->getRepository(Endeforn::class)->findOneBy($params);
            if ($endeforn !== null) {
                $this->entityManager->remove($endeforn);
            }

            $endeprospect = $this->entityManager->getRepository(Endeprospect::class)->findOneBy($params);
            if ($endeprospect !== null) {
                $this->entityManager->remove($endeprospect);
            }

            $endevende = $this->entityManager->getRepository(Endevende::class)->findOneBy($params);
            if ($endevende !== null) {
                $this->entityManager->remove($endevende);
            }

            $this->entityManager->flush();

            $enderecoparc = $this->entityManager->getRepository(Enderecoparc::class)->findOneBy($params);
            if ($enderecoparc !== null) {
                $this->entityManager->remove($enderecoparc);
            }

            $this->entityManager->flush();
            $this->entityManager->commit();
        } catch(\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }


    /*
     *
     *  Funções de validação de dados
     *
     */
    private function validateEnderecoParceiro($row) {
        if (empty($row["NRPARCNEGOCIO"])) {
            throw new \Exception("Número do parceiro de negócio não informado.");
            //var_dump(888,$row);die;
        }
        if (empty($row["CDTIPOENDERECO"])) {
            throw new \Exception("Tipo de endereço não informado.");
        }
        if (empty($row["CDPAIS"])) {
            throw new \Exception("País não informado.");
        }
        if (empty($row["SGESTADO"])) {
            throw new \Exception("Estado não informado.");
        }
        if (empty($row["CDMUNICIPIO"])) {
            throw new \Exception("Município não informado.");
        }
        if (empty($row["DSENDERECO"])) {
            throw new \Exception("Endereço não informado.");
        }
        if ($row["__is_new"] || $row["tipoEnderecoWasEdited"]) {
            $nrorg = $this->environment->getNrorgTrab();
            $nrparcnegocio  = $row["NRPARCNEGOCIO"];
            $cdtipoendereco = $row["CDTIPOENDERECO"];
            $enderecoparc   = $this->entityManager->getRepository(Enderecoparc::class)->findOneBy(array(
                "nrparcnegocio"  => $nrparcnegocio,
                "cdtipoendereco" => $cdtipoendereco,
                "nrorg"          => $nrorg
            ));

            if ($enderecoparc !== null) {
                throw new \Exception("Já existe um endereço cadastrado para o tipo selecionado.");
            }
        }
    }
}
